package ip;

import java.awt.*; 
import java.awt.event.*; 
import java.io.*; 

/** This class implements a simple dialog that lets a user specify a directory and file
    name for Saving or Opening. */
public class IPFileDialog extends FileDialog { 

/** The Subnet to be used.  For a Save, this is the Subnet to be saved.  For a Load
    this is the Subnet loaded and returned with getNetwork(). */
   private Subnet network=null;

/** The current directory. */
   File dir;
   
/** Create a FileDialog. The owner parameter is required for a Dialog.  net is
    the Subnet to be used for the operation.  isLoad is true for an 'Open' or
    false for a Save. */ 
   public IPFileDialog(Frame owner,Subnet net) {
      super(owner);
      network=net;
      dir=new File("");
      setDirectory(dir.getAbsolutePath());
      pack();
   }

/** Returns the Subnet used for the operation. */
   public Subnet getNetwork() {return network;}

/** Sets the Subnet used for the operation. */
   public void setNetwork(Subnet net) {network=net;}

/** If a Save, creates an IPWriter and saves the Subnet.  If a Load, creates an
    IPReader and creates a new Subnet from the file.  If a Load is unsuccessful, the
    FileDialog getNetwork() method will return null. */

   public void show() {
      setFile(network.getIPAddressString()+".txt");
      super.show();
      boolean valid=false;
      String fn=getFile();
      if(fn!=null) {
         fn=getDirectory()+fn;
         if(getMode()==FileDialog.SAVE) {
            try {
               IPWriter w=new IPWriter(fn);
               w.write(network);
            } catch (IOException ie) {System.out.println("IO Error\n"+ie);
            }
         } else if(getMode()==FileDialog.LOAD) {
            try {
               IPReader ipr=new IPReader(fn);
               network=ipr.readNet();
            } catch(IOException ie1) {
               System.out.println("IO Error\n"+ie1);
               network=null;
            }
         }
      }
   }

   public void setMode(int mode) {
      super.setMode(mode);
      if(mode==FileDialog.LOAD) setTitle("Load a Network");
      else setTitle("Save a Network");
   }
      
   public static void main(String[] args) {
      IPFileDialog fd=new IPFileDialog(new Frame(),new Subnet(0xc0010200,0xffffff00,0));
      fd.show();
      fd.setMode(FileDialog.LOAD);
      fd.show();
   }
} 
