package ip;

import java.awt.*; 
import java.awt.event.*; 
import java.io.*; 

public class NetworkFrame extends Frame implements ActionListener,WindowListener,KeyListener { 
   TextField countField=new TextField(4);
   Panel subnetPanel=new Panel();
   Panel netPanel=new Panel();
   Panel ipPanel=new Panel(new GridLayout(3,1,1,1));
   Panel allocPanel=new Panel(new GridLayout(3,1,1,1));
   Subnet network;
   Label aCount=new Label();
   Label addLabel=new Label();
   Label maskLabel=new Label();
   Label gateLabel=new Label();
   NetworkDialog parent=null;
   IPFileDialog fileDialog=null;

   public NetworkFrame(Subnet addr) {
      network=addr;
      init();
      addWindowListener(this);
   }

   public void init() {

      netPanel.setBackground(Color.darkGray);
      netPanel.setLayout(new BorderLayout(1,1));

      ipPanel.setBackground(Color.white);
      ipPanel.add(addLabel);
      ipPanel.add(maskLabel);
      ipPanel.add(gateLabel);
      initLabels();
      netPanel.add(ipPanel,BorderLayout.WEST);

      Button saveButton=new Button("Save");
      saveButton.addActionListener(this);
      Button loadButton=new Button("Open");
      loadButton.addActionListener(this);
      Button subnetButton=new Button("Allocate");
      subnetButton.addActionListener(this);
      Button newButton=new Button("New");
      newButton.addActionListener(this);
      Button quitButton=new Button("Quit");
      quitButton.addActionListener(this);

      Panel saveload=new Panel(new GridLayout(2,2));
      saveload.add(newButton);
      saveload.add(loadButton);
      saveload.add(quitButton);
      saveload.add(saveButton);
      Panel outer=new Panel();
      outer.setBackground(Color.white);
      outer.add(saveload,BorderLayout.CENTER);
      netPanel.add(outer,BorderLayout.CENTER);

      allocPanel.setBackground(Color.white);
      allocPanel.add(aCount);
      allocPanel.add(countField);
      allocPanel.add(subnetButton);
      netPanel.add(allocPanel,BorderLayout.EAST);

      subnetPanel.setLayout(new GridLayout(64,1,1,4));
      subnetPanel.setBackground(Color.darkGray);
      ScrollPane sc=new ScrollPane();
      sc.getVAdjustable().setUnitIncrement(24);
      sc.getVAdjustable().setBlockIncrement(96);
      sc.add(subnetPanel);
      setTitle("Network Subnet Tool");

      add(netPanel,BorderLayout.NORTH);
      add(sc,BorderLayout.CENTER);
      pack();
      setVisible(true);
      countField.requestFocus();
      countField.addKeyListener(this);
   }

  void initLabels() {
      aCount.setText(""+network.getCount());
      addLabel.setText(network.getIPAddressString());
      maskLabel.setText(network.getNetMaskString());
      gateLabel.setText(network.getDefaultGatewayString());
   }

   private void allocateNet() {
      try {
         int count=Integer.parseInt(countField.getText().trim());
         if(count>0) {
            if(count<4) count=4;
            Subnet sn=network.allocate(count);
            if(sn!=null) {
               subnetPanel.add(new SubnetPanel(sn));
               aCount.setText(""+(network.getCount()-sn.getCount()));
            }
            pack();
         }
      } catch(NumberFormatException nfe) {;}
   }

   public void actionPerformed(ActionEvent e) {
      String button=e.getActionCommand();
      if(button.equals("Save")) {
         if(fileDialog==null) fileDialog=new IPFileDialog(this,network);
         fileDialog.setMode(FileDialog.SAVE);
         fileDialog.setNetwork(network);
         fileDialog.show(); 
      } else if(button.equals("Allocate")) { 
         allocateNet();
      } else if(button.equals("New")) {
         NetworkDialog nd=new NetworkDialog(this);
         nd.setVisible(true);
         Subnet subnet=nd.getNetwork();
         if(subnet!=null) resetNet(subnet); 
      } else if(button.equals("Open")) {
         if(fileDialog==null) fileDialog=new IPFileDialog(this,network);
         fileDialog.setMode(FileDialog.LOAD);
         fileDialog.show(); 
         Subnet sn=fileDialog.getNetwork();
         if(sn!=null) {
            resetNet(sn);
            addAllocatedSubnets(sn);
            pack();
         }
      } else if(button.equals("Quit")) {
         setVisible(false);
         dispose();
      }
   }

   public Subnet getNetwork() {return network;}

   public void resetNet(Subnet sn) {
      network=sn;
      subnetPanel.removeAll();
      initLabels();
      pack();
   }

   public void addAllocatedSubnets(Subnet sn) {
      if(sn.hasSubnet()) {
         Subnet c0=sn.getChild0();
         Subnet c1=sn.getChild1();
         addAllocatedSubnets(c1);
         addAllocatedSubnets(c0);
         if(!c1.isAvailable()) subnetPanel.add(new SubnetPanel(c1));
         if(!c0.isAvailable()) subnetPanel.add(new SubnetPanel(c0));
      }
   }

   public void windowActivated(WindowEvent e) {;}

   public void windowClosed(WindowEvent e) {;}

   public void windowClosing(WindowEvent e) {
      setVisible(false);
      dispose();
   } 

   public void windowDeactivated(WindowEvent e) {;} 

   public void windowDeiconified(WindowEvent e) {;} 

   public void windowIconified(WindowEvent e) {;} 

   public void windowOpened(WindowEvent e) {;} 

   public void keyPressed(KeyEvent e) {;}

   public void keyTyped(KeyEvent e) {;}

   public void keyReleased(KeyEvent e) {
      if(e.getKeyCode()==KeyEvent.VK_ENTER) allocateNet();
   }

   public static void main(String[] args) {
      String addr="192.168.10.0";
      String mask="255.255.255.0";
      String gw="192.168.1.1";
      if(args.length>0) addr=args[0];
      if(args.length>1) mask=args[1];
      if(args.length>2) gw=args[2];

      NetworkFrame nf=new NetworkFrame(new Subnet(addr,mask,gw));
   }
} 
